@@ -2,6 +2,7 @@
 
 __author__ = 'Alexander.Li'
 
+import random
 import uuid
 import json
 import pollworker
@@ -11,6 +12,7 @@ from secp256k1py import secp256k1
 from errorbuster import formatError
 from .functions import Aws
 
+rd_num = random.randint(1000000, 9999999)
 
 def load_config():
     with open('.broker_config', 'r') as f:
@@ -132,45 +134,57 @@ class MRequest(object):
         return "req user:%s msg:%s" % (self.sender.user_id, self.message)
 
 
-def send_command_to(config, aws, target, response):
-    resp = response.make_response(config, aws, target.get('public_key'))
-    logging.info(aws.send_msg(target.get('endpoint'), json.dumps(resp)))
+def send_command_to(config, aws, endpoint, response):
+    resp = response.make_response(config, aws)
+    logging.info(aws.send_msg(endpoint, resp))
+    logging.info('command %s sended', response.data)
+    if isinstance(response, MessageResponse):
+        # 如果是Message消息就发APNS推送
+        msg = response.data.get('content').get('body').get('content')[:72]
+        resp2 = aws.send_sns_to(response.arn, msg)
+        logging.info('push notification sended:%s with arn:%s', resp2, response.arn)
 
 
 class Response(object):
-    def __init__(self, basic_dict):
+    def __init__(self, basic_dict, tartget):
         self.data = basic_dict
         self.new_version = False
+        self.pubkey = tartget['pubkey']
+        self.endpoint = tartget['endpoint']
+        self.arn = tartget['arn']
 
-    def set_version(self, new_version=False):
-        self.new_version = new_version
+    def set_pair(self, pubkey, endpoint):
+        self.pubkey = pubkey
+        self.endpoint = endpoint
+        return self
 
     def bin_resp(self, data, pk):
         data_arr = bytearray(pk.encode())
         data_arr.extend(data)
         return base64.urlsafe_b64encode(data_arr).decode('utf8')
 
-    def make_response(self, config, aws, target_public):
+    def make_response(self, config, aws):
         self.data.update(
             {
                 'sender': {
                     'host_id': 'miaomi',
                     'user_id': config.get('ID'),
                     'nick_name': config.get('NICK'),
-                    'public_key': config.get('BROKER_PUBLIC'),
-                    'endpoint': aws.with_url(config.get('ID')),
-                    'info': config.get('INFO')
+                    #'public_key': config.get('BROKER_PUBLIC'),
+                    #'endpoint': aws.with_url(config.get('ID')),
+                    'info': config.get('INFO'),
+                    'up_version': rd_num
                 }
             }
         )
         keypair = secp256k1.make_keypair()
-        targetPublicKey = secp256k1.PublicKey.restore(target_public)
-        enced = targetPublicKey.encrypt(keypair.privateKey, json.dumps(self.data).encode(), raw=self.new_version)
+        targetPublicKey = secp256k1.PublicKey.restore(self.pubkey)
+        enced = targetPublicKey.encrypt(keypair.privateKey, json.dumps(self.data).encode(), raw=True)
         return self.bin_resp(enced['enc'], str(keypair.publicKey))
 
 
 class MessageResponse(Response):
-    def __init__(self, text, _new_version=False):
+    def __init__(self, text, target):
         message = {
             'content': {
                 'unique_id': uuid.uuid4().hex,
@@ -181,22 +195,39 @@ class MessageResponse(Response):
                 }
             }
         }
-        super(MessageResponse, self).__init__(message)
+        super(MessageResponse, self).__init__(message, target)
+
+
+class BroadCastResponse(Response):
+    def __init__(self, text, sender_id, target):
+        message = {
+            'content': {
+                'unique_id': uuid.uuid4().hex,
+                'content-type': 'broadcast',
+                'action': 'new',
+                'body': {
+                    'sender_id': sender_id,
+                    'content': text
+                }
+            }
+        }
+        super(BroadCastResponse, self).__init__(message, target)
+
 
 
 class CommandResponse(Response):
-    def __init__(self, name, *params, _new_version=False):
+    def __init__(self, name, *params, target=None):
         command = {
             'command': {
                 'name': name,
                 'params': params
             }
         }
-        super(CommandResponse, self).__init__(command)
+        super(CommandResponse, self).__init__(command, target)
 
 
 class ContackResponse(Response):
-    def __init__(self, users, _new_version=False):
+    def __init__(self, users, target):
         message = {
             'content': {
                 'unique_id': uuid.uuid4().hex,
@@ -213,7 +244,7 @@ class ContackResponse(Response):
                 }
             }
         }
-        super(ContackResponse, self).__init__(message)
+        super(ContackResponse, self).__init__(message, target)
 
 
 executors = {}
@@ -252,7 +283,7 @@ def process_message(config, message):
         publicKey = secp256k1.PublicKey.restore(pk.decode())
         try:
             raw = privateKey.decrypt(publicKey, bytes(bytes_data), None)
-            logging.info(f"get message:${raw}")
+            logging.info(f"get message:${raw[:100]}")
             req = MRequest(raw, new_version=True)
         except Exception as e:
             logging.error(formatError(e))
@@ -276,13 +307,9 @@ def worker(pid, message):
                 if MESSAGE_HANDLER in executors:
                     resp = executors[MESSAGE_HANDLER](config, aws, req)
         if resp:
-            resp.set_version(req.new_version)
-            if req.new_version:
-                response_text = resp.make_response(config, aws, req.sender.public_key)
-            else:
-                response_text = json.dumps(resp.make_response(config, aws, req.sender.public_key))
-            aws.send_msg(req.sender.endpoint, response_text)
-            logging.info(f'response:{response_text}')
+            response_text = resp.make_response(config, aws)
+            aws.send_msg(resp.endpoint, response_text)
+            logging.info(f'response:{resp.data}')
             logging.info('message: sended!')
 
 
